/***************************************************
* By XC8 compiler
*  File Name: lux_talking_meter.c
*  Description: talking lux meter
*  RC2 out /reset for ATP
*  RC4 out RXD for ATP
*  RC3 in PLAY
*  RC5 in SETTING
*  RC0 out SENSOR I2C SCL
*  RC1 inout SENSOR I2C SDA
*  RA4 in AD threshold (0 to 5V)
*  COPYRIGHT 2016 MYCOMKITS.COM, owned by CNET LIMITED
*  $BEv%W%m%0%i%`$NCx:n8"$O!"@=:n<T!V%^%$%3%s%-%C%H%I%C%H%3%`1?1D!!M-8B2q<R%/%M%C%H!W$K5"B0$7$^$9!#(B
*  $BCx:n8"$rJ|4~$7$F$$$^$;$s$,!"Ev%W%m%0%i%`$r;H$C$?3X=,$NCf$G%W%m%0%i%`$r<+M3$KJQ99$7$F$*;H$$$/$@$5$$!#(B
***************************************************/


#include <xc.h>

#define _XTAL_FREQ  8000000  // Clock Frequency 8MHz

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is MCLR)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config CPD = OFF        // Data Memory Code Protection (Data memory code protection is disabled)
#pragma config BOREN = ON       // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover (Internal/External Switchover mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL enabled)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (Low-voltage programming enabled)


extern void I2CMasterSendMsg(unsigned char adrs, unsigned char *msg, unsigned char cnt);
extern void I2CMasterRecvMsg(unsigned char adrs, unsigned char *msg, unsigned char cnt);

unsigned char recvMsg[4];
unsigned char sendMsg[4];

extern void TSL2561PowerUp(void);
extern void TSL2561GetData(unsigned char ctrl);
extern unsigned int CalculateLux(unsigned int iGain, unsigned int tInt, unsigned int
									ch0, unsigned int ch1, int iType);
extern void TalkLux(void);

unsigned int luxGain;
unsigned int luxTime;
unsigned int ch0Data, ch1Data;
unsigned int luxVal;

extern void SerialTransmitChar(char str);
extern void SerialTransmitString(char *str);
extern void SerialTransmitNumber(unsigned int val);

unsigned char fig[5];

unsigned char adData;
unsigned short interval;    // interval = time(sec)*122

/*
 * Main Routine
 */
int main(int argc, char** argv) {

    // Clock Setup
    OSCCON = 0x70;  // 8MHz
  
    //I/O Port Setup
    OPTION_REGbits.nWPUEN = 0;  // Weak pullup enabled
    TRISA = 0x3F;   // All Input
    ANSELA = 0x10;	// RA4 Analog input
    WPUA = 0x2F;    // RA5,RA3-RA0 pullup
    TRISC = 0x3B;	// RC2 Output
    ANSELC = 0x00;  // All Digital
    WPUC = 0x28;    // RC5,RC3 pullup
    
    // MSSP(I2C) Setup
    SSP1ADD = 0x13;     // 100kHz@8MHz
    SSP1CON1 = 0x38;    // I2C Enable, Mater Mode
    SSP1STATbits.CKE = 1;   // Thresholds compliant with SMBus 
 
    // EUSART(RS232C) Setup
    TXSTA = 0x20;   // 8-bit, Transmit enabled, low Speed baud rate
    RCSTA = 0x80;   // Serial port enable
    BAUDCON = 0x08; // 16-bit Baud Rate
    SPBRG = 51;     // 9600bps 8MHz/51 
   
    // ADC Setup
    ADCON0 = 0x0D;      // AN3(RA4), ADC Enable
    ADCON1 = 0x50;      // Left justified, Fosc/16(2.0us), VDD
    
    // Timer0 Setup
    //      Timer0 overflow 30.517Hz(0.032768s)
    OPTION_REGbits.TMR0CS = 0;  // Fosc/4
    OPTION_REGbits.PSA = 0; // Prescaler
    OPTION_REGbits.PS = 7;  // Prescaaler 1:256
    TMR0 = 0;               //
    
    // Timer1 Setup
    T1CON = 0x41;       // Fosc, 1:1, Enables Timer1
    T1GCON = 0xC1;      // Gate enabled, Timer0 overflow
    TMR1L = 0x3B;       // 10s 0xffff - 122*10s (122 is 30.517*4)
    TMR1H = 0xFB;       // 10s
    PIR1bits.TMR1IF = 0;   // Timer1 Flag Clear 
    interval = 0xFB3B;  // 10sec
    
    // ATP3011 RESET
    PORTCbits.RC2 = 1;            // RESET Clear
    PORTCbits.RC2 = 0;            // RESET
    __delay_ms(500);
    PORTCbits.RC2 = 1;            // RESET Clear
    __delay_ms(100);    // Wait ATP3011 Startup (>80ms)
    
    // ATP3011 Speech Synthesaizer
    SerialTransmitString("?");     // Set Baud Rate
    __delay_ms(100);
    SerialTransmitString("to-kinngu/ru'kkusu/me'-ta-/kido'-shima'shita\r");
    __delay_ms(4000);
    
    // TSL2561 power up
    TSL2561PowerUp();

    //
    // LUX Measure
    //
    while(1){
        if(PORTCbits.RC5 == 0){ // Setting SW Pushed
            ADCON0bits.GO_nDONE = 1;    // AD start
            __delay_us(100);
            adData = ADRESH;
            if(adData < 6) adData = 6;
            adData >>= 1;   // 1/2
            interval = 0xffff - adData*122;
            TMR1L = interval && 0x00FF;
            TMR1H = interval >> 8;
            PIR1bits.TMR1IF = 0;   // Timer1 Flag Clear
            if (adData > 120){
                SerialTransmitString("$ripi'-to/sima'sen\r");
            } else {
                SerialTransmitString("$<NUMK VAL=");
                SerialTransmitNumber(adData);
                SerialTransmitString("> biyou\r");
            }
            __delay_ms(3000);
       } else if(PIR1bits.TMR1IF == 1 && adData <= 120){   // Timer1
            TMR1L = interval && 0x00FF;
            TMR1H = interval >> 8;
            PIR1bits.TMR1IF = 0;   // Timer1 Flag Clear
            TalkLux();
           __delay_ms(2000);
        } else if(PORTCbits.RC3 == 0){ // PLAY SW Pushed
            TalkLux();
           __delay_ms(2000);
        }
    }
}

//
// TSL2561 Control Functions
//
void TalkLux(void){

    luxGain = 1;	// Gain 16x
    luxTime = 1;	// INTEG 101ms
    TSL2561GetData(0x11);  // Gain 16x, INTEG 101ms
    if(ch0Data > 0x8000 || ch1Data > 0x8000){ // if data is large, gain down
        luxGain = 0;	// Gain 1x;
        luxTime = 1;	// INTEG 101ms
        TSL2561GetData(0x01);  // Gain 1x, INTEG 101ms
        if(ch0Data > 0x9000 || ch1Data > 0x9000){ // too large
            SerialTransmitString("$sokuteidekimasen\r");
            return;
        }
    }

    luxVal = CalculateLux(luxGain,        // iGain - gain, where 0:1X, 1:16X
                           	luxTime,		// tInt - integration time, where 0:13.7mS, 1:101mS, 2:402mS,3:Manual
           					ch0Data,	// raw channel value from channel 0 of TSL2560
               				ch1Data,	// raw channel value from channel 1 of TSL2560
                   			0);			// iType - package type (T or CS)
 
   	SerialTransmitString("$<NUMK VAL=");
   	SerialTransmitNumber(luxVal);
   	SerialTransmitString("> ru'kkusu\r");

}

void TSL2561PowerUp(void){
    sendMsg[0] = 0x80;  // Control Register
    sendMsg[1] = 0x03;  // Powerup
    I2CMasterSendMsg(0x39,sendMsg,2);

    sendMsg[0] = 0x81;  // Timing Register
    sendMsg[1] = 0x11;  // Gain x16, 101ms 
    I2CMasterSendMsg(0x39,sendMsg,2);
}

void TSL2561GetData(unsigned char ctrl){
    
    sendMsg[0] = 0x81;  // Timing Register
    sendMsg[1] = ctrl;
    I2CMasterSendMsg(0x39,sendMsg,2);

    __delay_ms(250);    // Wait Measuring Time 101ms*2+margin
    
	sendMsg[0] = 0x8C;  // CH0 Register
	I2CMasterSendMsg(0x39,sendMsg,1);
	I2CMasterRecvMsg(0x39,recvMsg,2);
	ch0Data = (((unsigned int)recvMsg[1] << 8) & 0xFF00) | ((unsigned int)recvMsg[0] & 0x00FF);
	
	sendMsg[0] = 0x8E;  // CH1 Register
	I2CMasterSendMsg(0x39,sendMsg,1);
	I2CMasterRecvMsg(0x39,recvMsg,2);
	ch1Data = (((unsigned int)recvMsg[1] << 8) & 0xFF00) | ((unsigned int)recvMsg[0] & 0x00FF);
}

//
// RS232C Functiion
//
void SerialTransmitChar(char str){
	while(TXSTAbits.TRMT == 0);     // TSR Full
    TXREG = str;
}

void SerialTransmitString(char *str){
    int i;
    
    i = 0;
    while(str[i] != 0x00){
        SerialTransmitChar(str[i]);
        i++;
    }
}

void SerialTransmitNumber(unsigned int val){
	unsigned long temp;
	int i;
	
	temp = val;
	
	for(i=0;i<5;i++){
		fig[i] = temp%10;
		temp = temp/10;
	}

    SerialTransmitChar(fig[4] | 0x30);
    SerialTransmitChar(fig[3] | 0x30);
    SerialTransmitChar(fig[2] | 0x30);
    SerialTransmitChar(fig[1] | 0x30);
    SerialTransmitChar(fig[0] | 0x30);
}

//
// I2C Control Functions
//
extern void I2CWaitIdle(void);
extern void I2CMasterSendChar(unsigned char data);
extern unsigned char I2CMasterRecvChar(unsigned char);

void I2CWaitIdle(void){
    while(1){
        if(SSP1CON2bits.SEN == 0                    // Start condition idle
                && SSP1CON2bits.RSEN == 0           // Repeated Start condition idle
                && SSP1CON2bits.PEN == 0            // Stop condition idle
                && SSP1CON2bits.RCEN == 0           // Receive idle
                && SSP1CON2bits.ACKEN == 0          // Acknowledge sequence idle
                && SSP1STATbits.R_nW == 0) break;   // Transmit is not in progress
    }
}

void I2CMasterSendChar(unsigned char data){
    SSP1BUF = data;
    while(SSP1STATbits.BF);
    while(SSP1CON2bits.ACKSTAT);
}
    
void I2CMasterSendMsg(unsigned char adrs, unsigned char *msg, unsigned char cnt){
    unsigned char i;
    
    I2CWaitIdle();

    SSP1CON2bits.SEN = 1;       //Start Condition Enable
    while(SSP1CON2bits.SEN);    // Wait StartCondition Idle

    I2CMasterSendChar((adrs << 1) & 0xFE);  // Adrs + Write
    for(i=0;i<cnt;i++)
        I2CMasterSendChar(msg[i]);

    SSP1CON2bits.PEN = 1;       // Stop Condition Enable
    while(SSP1CON2bits.PEN);    // Wait Stop Condition Idle
}

unsigned char I2CMasterRecvChar(unsigned char ans){
    unsigned char data;
    
    SSP1CON2bits.RCEN =1;       // Receive enable
    while(!SSP1STATbits.BF);    // Wait Receive complete
    SSP1CON2bits.ACKDT = ans;
    SSP1CON2bits.ACKEN = 1;     // Initiate Acknowledge sequence
    data = SSP1BUF;
    SSP1CON1bits.SSPOV = 0;     // Clear Receive overflow indicater
    while(SSP1CON2bits.ACKEN);  // Wait Acknowledge sequence idle
    
    return(data);
}
    
void I2CMasterRecvMsg(unsigned char adrs, unsigned char *msg, unsigned char cnt){
    unsigned char i;
    
    I2CWaitIdle();

    SSP1CON2bits.SEN = 1;       //Start Condition Enable
    while(SSP1CON2bits.SEN);    // Wait StartCondition Idle
    
    I2CMasterSendChar((adrs << 1) | 0x01);  // Adrs + Read
    for(i=0;i<(cnt-1);i++){
        msg[i] = I2CMasterRecvChar(0);
    }
    msg[i] = I2CMasterRecvChar(1);
    
    SSP1CON2bits.PEN = 1;       // Stop Condition Enable
    while(SSP1CON2bits.PEN);    // Wait Stop Condition Idle    
}


//****************************************************************************
//
// Copyright  2004?2005 TAOS, Inc.
//
// THIS CODE AND INFORMATION IS PROVIDED $B!)(BHhAS IS$B!)(BHh WITHOUT WARRANTY OF ANY
// KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
// PURPOSE.
//
// Module Name:
// lux.cpp
//
//****************************************************************************

#define LUX_SCALE 14		// scale by 2^14
#define RATIO_SCALE 9		// scale ratio by 2^9

//---------------------------------------------------
// Integration time scaling factors
//---------------------------------------------------

#define CH_SCALE 10				// scale channel values by 2^10
#define CHSCALE_TINT0 0x7517	// 322/11 * 2^CH_SCALE
#define CHSCALE_TINT1 0x0fe7	// 322/81 * 2^CH_SCALE

//---------------------------------------------------
// T, FN, and CL Package coefficients
//---------------------------------------------------
// For Ch1/Ch0=0.00 to 0.50
// 			Lux/Ch0=0.0304?0.062*((Ch1/Ch0)^1.4)
// 			piecewise approximation
// 				For Ch1/Ch0=0.00 to 0.125:
// 					Lux/Ch0=0.0304?0.0272*(Ch1/Ch0)
//
// 				For Ch1/Ch0=0.125 to 0.250:
// 					Lux/Ch0=0.0325?0.0440*(Ch1/Ch0)
//
// 				For Ch1/Ch0=0.250 to 0.375:
// 					Lux/Ch0=0.0351?0.0544*(Ch1/Ch0)
//
// 				For Ch1/Ch0=0.375 to 0.50:
// 					Lux/Ch0=0.0381?0.0624*(Ch1/Ch0)
//
// For Ch1/Ch0=0.50 to 0.61:
// 			Lux/Ch0=0.0224?0.031*(Ch1/Ch0)
//
// For Ch1/Ch0=0.61 to 0.80:
// 			Lux/Ch0=0.0128?0.0153*(Ch1/Ch0)
//
// For Ch1/Ch0=0.80 to 1.30:
// 			Lux/Ch0=0.00146?0.00112*(Ch1/Ch0)
//
// For Ch1/Ch0>1.3:
// 			Lux/Ch0=0
//---------------------------------------------------
#define K1T 0x0040		// 0.125 * 2^RATIO_SCALE
#define B1T 0x01f2		// 0.0304 * 2^LUX_SCALE
#define M1T 0x01be		// 0.0272 * 2^LUX_SCALE

#define K2T 0x0080		// 0.250 * 2^RATIO_SCALE
#define B2T 0x0214		// 0.0325 * 2^LUX_SCALE
#define M2T 0x02d1		// 0.0440 * 2^LUX_SCALE

#define K3T 0x00c0		// 0.375 * 2^RATIO_SCALE
#define B3T 0x023f		// 0.0351 * 2^LUX_SCALE
#define M3T 0x037b		// 0.0544 * 2^LUX_SCALE

#define K4T 0x0100		// 0.50 * 2^RATIO_SCALE
#define B4T 0x0270		// 0.0381 * 2^LUX_SCALE
#define M4T 0x03fe		// 0.0624 * 2^LUX_SCALE

#define K5T 0x0138		// 0.61 * 2^RATIO_SCALE
#define B5T 0x016f		// 0.0224 * 2^LUX_SCALE
#define M5T 0x01fc		// 0.0310 * 2^LUX_SCALE

#define K6T 0x019a		// 0.80 * 2^RATIO_SCALE
#define B6T 0x00d2		// 0.0128 * 2^LUX_SCALE
#define M6T 0x00fb		// 0.0153 * 2^LUX_SCALE

#define K7T 0x029a		// 1.3 * 2^RATIO_SCALE
#define B7T 0x0018		// 0.00146 * 2^LUX_SCALE
#define M7T 0x0012		// 0.00112 * 2^LUX_SCALE

#define K8T 0x029a		// 1.3 * 2^RATIO_SCALE
#define B8T 0x0000		// 0.000 * 2^LUX_SCALE
#define M8T 0x0000		// 0.000 * 2^LUX_SCALE

//---------------------------------------------------
// CS package coefficients
//---------------------------------------------------
// For 0 <= Ch1/Ch0 <= 0.52
// 			Lux/Ch0 = 0.0315?0.0593*((Ch1/Ch0)^1.4)
// 			piecewise approximation
// 				For 0 <= Ch1/Ch0 <= 0.13
// 					Lux/Ch0 = 0.0315?0.0262*(Ch1/Ch0)
// 				For 0.13 <= Ch1/Ch0 <= 0.26
// 					Lux/Ch0 = 0.0337?0.0430*(Ch1/Ch0)
// 				For 0.26 <= Ch1/Ch0 <= 0.39
// 					Lux/Ch0 = 0.0363?0.0529*(Ch1/Ch0)
// 				For 0.39 <= Ch1/Ch0 <= 0.52
// 					Lux/Ch0 = 0.0392?0.0605*(Ch1/Ch0)
// For 0.52 < Ch1/Ch0 <= 0.65
// 		Lux/Ch0 = 0.0229?0.0291*(Ch1/Ch0)
// For 0.65 < Ch1/Ch0 <= 0.80
// 		Lux/Ch0 = 0.00157?0.00180*(Ch1/Ch0)
// For 0.80 < Ch1/Ch0 <= 1.30
// 		Lux/Ch0 = 0.00338?0.00260*(Ch1/Ch0)
// For Ch1/Ch0 > 1.30
// 		Lux = 0
//---------------------------------------------------
#define K1C 0x0043		// 0.130 * 2^RATIO_SCALE
#define B1C 0x0204		// 0.0315 * 2^LUX_SCALE
#define M1C 0x01ad		// 0.0262 * 2^LUX_SCALE

#define K2C 0x0085		// 0.260 * 2^RATIO_SCALE
#define B2C 0x0228		// 0.0337 * 2^LUX_SCALE
#define M2C 0x02c1		// 0.0430 * 2^LUX_SCALE

#define K3C 0x00c8		// 0.390 * 2^RATIO_SCALE
#define B3C 0x0253		// 0.0363 * 2^LUX_SCALE
#define M3C 0x0363		// 0.0529 * 2^LUX_SCALE

#define K4C 0x010a		// 0.520 * 2^RATIO_SCALE
#define B4C 0x0282		// 0.0392 * 2^LUX_SCALE
#define M4C 0x03df		// 0.0605 * 2^LUX_SCALE

#define K5C 0x014d		// 0.65 * 2^RATIO_SCALE
#define B5C 0x0177		// 0.0229 * 2^LUX_SCALE
#define M5C 0x01dd		// 0.0291 * 2^LUX_SCALE

#define K6C 0x019a		// 0.80 * 2^RATIO_SCALE
#define B6C 0x0101		// 0.0157 * 2^LUX_SCALE
#define M6C 0x0127		// 0.0180 * 2^LUX_SCALE

#define K7C 0x029a		// 1.3 * 2^RATIO_SCALE
#define B7C 0x0037		// 0.00338 * 2^LUX_SCALE
#define M7C 0x002b		// 0.00260 * 2^LUX_SCALE

#define K8C 0x029a		// 1.3 * 2^RATIO_SCALE
#define B8C 0x0000		// 0.000 * 2^LUX_SCALE
#define M8C 0x0000		// 0.000 * 2^LUX_SCALE


// lux equation approximation without floating point calculations
//////////////////////////////////////////////////////////////////////////////
// Routine: 	unsigned int CalculateLux(unsigned int ch0, unsigned int ch0, int iType)
//
// Description:	Calculate the approximate illuminance (lux) given the raw
// 				channel values of the TSL2560. The equation if implemented
// 				as a piece?wise linear approximation.
//
// Arguments:	unsigned int iGain - gain, where 0:1X, 1:16X
// 				unsigned int tInt - integration time, where 0:13.7mS, 1:100mS, 2:402mS,
// 				3:Manual
// 				unsigned int ch0 - raw channel value from channel 0 of TSL2560
// 				unsigned int ch1 - raw channel value from channel 1 of TSL2560
// 				unsigned int iType - package type (T or CS)
//
// Return: 		unsigned int - the approximate illuminance (lux)
//
//////////////////////////////////////////////////////////////////////////////

unsigned int CalculateLux(unsigned int iGain, unsigned int tInt, unsigned int ch0,
	unsigned int ch1, int iType)
{
	//---------------------------------------------------
	// first, scale the channel values depending on the gain and integration time
	// 16X, 402mS is nominal.
	// scale if integration time is NOT 402 msec
	unsigned long chScale;
	unsigned long channel1;
	unsigned long channel0;
	switch (tInt)
	{
		case 0: // 13.7 msec
			chScale = CHSCALE_TINT0;
			break;
		case 1: // 101 msec
			chScale = CHSCALE_TINT1;
			break;
		default: // assume no scaling
			chScale = (1 << CH_SCALE);
		break;
	}

	// scale if gain is NOT 16X
	if (!iGain) chScale = chScale << 4; // scale 1X to 16X

	// scale the channel values
	channel0 = (ch0 * chScale) >> CH_SCALE;
	channel1 = (ch1 * chScale) >> CH_SCALE;
	//------------------------------------------------------------------------

	// find the ratio of the channel values (Channel1/Channel0)
	// protect against divide by zero
	unsigned long ratio1 = 0;
	if (channel0 != 0) ratio1 = (channel1 << (RATIO_SCALE+1)) / channel0;

	// round the ratio value
	unsigned long ratio = (ratio1 + 1) >> 1;

	// is ratio <= eachBreak ?
	unsigned int b, m;
	switch (iType)
	{
		case 0:		// T, FN and CL package
			if ((ratio >= 0) && (ratio <= K1T))
				{b=B1T; m=M1T;}
			else if (ratio <= K2T)
				{b=B2T; m=M2T;}
			else if (ratio <= K3T)
				{b=B3T; m=M3T;}
			else if (ratio <= K4T)
				{b=B4T; m=M4T;}
			else if (ratio <= K5T)
				{b=B5T; m=M5T;}
			else if (ratio <= K6T)
				{b=B6T; m=M6T;}
			else if (ratio <= K7T)
				{b=B7T; m=M7T;}
			else if (ratio > K8T)
				{b=B8T; m=M8T;}
			break;

		case 1:// CS package
			if ((ratio >= 0) && (ratio <= K1C))
				{b=B1C; m=M1C;}
			else if (ratio <= K2C)
				{b=B2C; m=M2C;}
			else if (ratio <= K3C)
				{b=B3C; m=M3C;}
			else if (ratio <= K4C)
				{b=B4C; m=M4C;}
			else if (ratio <= K5C)
				{b=B5C; m=M5C;}
			else if (ratio <= K6C)
				{b=B6C; m=M6C;}
			else if (ratio <= K7C)
				{b=B7C; m=M7C;}
			else if (ratio > K8C)
				{b=B8C; m=M8C;}
			break;
	}

	unsigned long temp;
	temp = ((channel0 * b) - (channel1 * m));

	// do not allow negative lux value
	if (temp < 0) temp = 0;

	// round lsb (2^(LUX_SCALE-1))
	temp += (1 << (LUX_SCALE-1));

	// strip off fractional portion
	unsigned long lux = temp >> LUX_SCALE;

	return(lux);
}